Unit BoxFiles;

//Version 1.1
// -Added preservation of Comments(;....) after a normal .Box Entry
// -Whole-line Comments (Starting with an ";") aren't evaluated anymore
// Version 1.0

interface
Uses
  Windows,
  BoxAPI;

Procedure BoxReset(BoxPtr: Pointer);
Function BoxReadNextConfig(BoxPtr: Pointer; szPrefix: String): String;

Function GetBoxString(BoxPtr: Pointer; KeyName: String; Default: String): String;
Function GetBoxInt(BoxPtr: Pointer; KeyName: String; Default: Integer): Integer;
Function GetBoxBool(BoxPtr: Pointer; KeyName: String): Boolean;

Procedure SetBoxString(BoxPtr: Pointer; KeyName: String; Str2Set: String);
Procedure SetBoxInt(BoxPtr: Pointer; KeyName: String; value: Integer);

Function OpenBox(FileName: String;SubBox: String): Pointer;
Function CloseBox(BoxPtr: pointer; Save: Boolean): Pointer;

implementation

Type
  PBoxEntry = ^TBoxEntry;
  TBoxEntry = packed Record
                Content,Key,Comment: String;
                Next: PBoxEntry;
              End;

  PSubBox = ^TSubBox;
  TSubBox = packed Record
              SubBoxPos: Pointer;
              First_Entry,
              Last_Entry: PBoxEntry;
            End;

  PBoxHeader = ^TBoxHeader;
  TBoxHeader = packed Record
                 FileName: String;
                 BoxPos: Pointer;
                 First_Entry,
                 Last_Entry: PBoxEntry;
               End;

Var
  BoxHeader: PBoxHeader;
  BoxEntry: PBoxEntry;
  BoxFile: TextFile;

function Trim(const S: string): string;
var
  I, L: Integer;
begin
  L := Length(S);
  I := 1;
  while (I <= L) and (S[I] <= ' ') do Inc(I);
  if I > L then Result := '' else
  begin
    while S[L] <= ' ' do Dec(L);
    Result := Copy(S, I, L - I + 1);
  end;
end;

Function GetComment(Entry: String):String;
Var
  i: integer;
  Comment: String;
Begin
  Comment := '';
  i := Length(Entry);
  While ( i>1 )AND( Comment='' ) Do
  Begin
    If ( Entry[i]=';' ) Then
      Comment := Copy(Entry,i,255);
    Dec(i);
  End;
  GetComment := Comment
End;

Procedure BoxReset(BoxPtr: Pointer);
Begin
  BoxHeader := BoxPtr;
  BoxHeader^.BoxPos := BoxHeader^.First_Entry;
End;

Function BoxReadNextConfig(BoxPtr: Pointer; szPrefix: String): String;
Begin
  BoxHeader := BoxPtr;
  BoxEntry := BoxHeader^.BoxPos;
  While (BoxEntry<>NIL)AND(UpStr(szPrefix)<>UpStr(Copy(Trim(BoxEntry^.Content),1,Length(szPrefix)))) DO
    BoxEntry := BoxEntry^.Next;
  If (BoxEntry <> Nil) Then
  Begin
    BoxHeader^.BoxPos := BoxEntry^.Next;
    Result := Trim(Copy(BoxEntry^.Content,Length(szPrefix)+1,255));
  End
  else
  Begin
    BoxHeader^.BoxPos := BoxHeader^.First_Entry;
    Result := '';
  End;
End;

Function GetBoxString(BoxPtr: Pointer; KeyName: String; Default: String): String;
Begin
  BoxHeader := BoxPtr;
  BoxEntry := BoxHeader^.First_Entry;
  While (BoxEntry<>NIL)AND(UpStr(KeyName)<>BoxEntry^.Key) DO
    BoxEntry := BoxEntry^.Next;
  If (BoxEntry <> Nil) Then
    Result := Trim(Copy(BoxEntry^.Content,Length(KeyName)+1,255))
  else
    Result := Default;
End;

Function GetBoxInt(BoxPtr: Pointer; KeyName: String; Default: Integer): Integer;
Var
  Str: String;

Begin
  Str := GetBoxString(BoxPtr, KeyName, 'NopE');
  If (Str <> 'NopE') Then
    Result := Str2Int(Str)
  else
    Result := Default;
End;

Function GetBoxBool(BoxPtr: Pointer; KeyName: String): Boolean;
Begin
  Result := GetBoxString(BoxPtr, KeyName, 'NopE')<> 'NopE';
End;

Procedure SetBoxString(BoxPtr: Pointer; KeyName: String; Str2Set: String);
Begin
  BoxHeader := BoxPtr;
  BoxEntry := BoxHeader^.First_Entry;
  While (BoxEntry<>NIL)AND(UpStr(KeyName)<>BoxEntry^.Key) DO
    BoxEntry := BoxEntry^.Next;
  If (BoxEntry <> Nil) Then
    BoxEntry^.Content := KeyName+#32+Str2Set+#32+BoxEntry^.Comment
  Else
  Begin
    New(BoxHeader^.Last_Entry^.next);
    BoxHeader^.Last_Entry := BoxHeader^.Last_Entry^.next;
    BoxHeader^.Last_Entry^.Content := KeyName+#32+Str2Set;
    BoxHeader^.Last_Entry^.Next := Nil;
  End;
End;

Procedure SetBoxInt(BoxPtr: Pointer; KeyName: String; value: Integer);
Begin
  SetBoxString(BoxPtr,KeyName,Int2Str(Value));
End;

Function OpenBox(FileName: String;SubBox: String): Pointer;
Begin
  AssignFile(BoxFile, FileName);
  {$I-}ReSet(BoxFile);{$I+}
  If (IOResult <> 0) then
    Result := nil
  Else Begin

    New(BoxHeader);
    Boxheader^.FileName := FileName;
    New(BoxEntry);
    Boxheader^.First_Entry := BoxEntry;
    Result := BoxHeader;

    While NOT(EOF(BoxFile))Do
    Begin
      Readln(BoxFile, BoxEntry^.Content);
      BoxEntry^.Key := '';
      If (Trim( BoxEntry^.Content )<>'')AND(Trim( BoxEntry^.Content[1] )<>';' ) Then
       Begin
         BoxEntry^.Key := UpStr(GetToken(BoxEntry^.Content,1));
         BoxEntry^.Comment := GetComment(BoxEntry^.Content);
         If ( BoxEntry^.Comment<>'' ) then
           BoxEntry^.Content := Trim( Copy( BoxEntry^.Content,1,length(BoxEntry^.Content)-length(BoxEntry^.Comment)))
       End;
      Boxheader^.Last_Entry := BoxEntry;
      New(BoxEntry^.Next);
      BoxEntry := BoxEntry^.Next;
    End;

    Dispose(BoxEntry);
    BoxHeader^.Last_Entry^.Next := Nil;
    BoxHeader^.BoxPos := Boxheader^.First_Entry;
    CloseFile(BoxFile);
  End;
End;

Function CloseBox(BoxPtr: pointer; Save: Boolean): Pointer;
Begin
  BoxHeader := BoxPtr;
  BoxEntry := Boxheader^.First_Entry;

  If Save then
  Begin
    AssignFile(BoxFile, BoxHeader^.FileName);
    ReWrite(BoxFile);

    While Not(BoxEntry = nil)Do
    Begin
      Boxheader^.First_Entry := BoxEntry^.Next;
      Writeln(BoxFile,BoxEntry^.Content);
      Dispose(BoxEntry);
      BoxEntry := Boxheader^.First_Entry;
    End;

    CloseFile(BoxFile);
  End
  Else Begin

    While Not(BoxEntry = nil)Do
    Begin
      Boxheader^.First_Entry := BoxEntry^.Next;
      Dispose(BoxEntry);
      BoxEntry := Boxheader^.First_Entry;
    End;

  End;

  CloseBox := nil;
  Dispose(BoxHeader);
End;

end.
