unit BoxFile2;

interface
Uses
  Windows,
  LsApi;

Procedure BoxReset(BoxPtr: Pointer);
Function BoxReadNextConfig(BoxPtr: Pointer; szPrefix: String): String;

Function GetBoxString(BoxPtr: Pointer; KeyName: String; Default: String): String;
Function GetBoxInt(BoxPtr: Pointer; KeyName: String; Default: Integer): Integer;
Function GetBoxBool(BoxPtr: Pointer; KeyName: String): Boolean;

Function OpenBox(FileName: String;SubBoxStr: String): Pointer;
Procedure CloseBox(BoxPtr: pointer; Save: Boolean);

implementation

Type
  PBoxEntry = ^TBoxEntry;
  TBoxEntry = packed Record
                Content: String;
                Line: Integer;
                Next: PBoxEntry;
              End;

  PBoxHeader = ^TBoxHeader;
  TBoxHeader = packed Record
                 FileName: String;
                 BoxPos: Pointer;
                 First_Entry: PBoxEntry;
               End;

Var
  BoxHeader: PBoxHeader;
  BoxEntry: PBoxEntry;
  BoxFile: TextFile;

Function Trim(const S: string): string;
var
  I, L: Integer;
begin
  L := Length(S);
  I := 1;
  while (I <= L) and (S[I] <= ' ') do Inc(I);
  if I > L then Result := '' else
  begin
    while S[L] <= ' ' do Dec(L);
    Result := Copy(S, I, L - I + 1);
  end;
end;

Function UpStr(Str: String): String;
Var
  Pos: Integer;
  UStr: String;
Begin
  UStr := Str;
  For Pos := 1 to length(UStr) Do
    if (UStr[pos] >= 'a') and (UStr[pos] <= 'z') then Dec(UStr[pos], 32);
  Result := UStr;
End;

Procedure BoxReset(BoxPtr: Pointer);
Begin
  BoxHeader := BoxPtr;
  BoxHeader^.BoxPos := BoxHeader^.First_Entry;
End;

Function BoxReadNextConfig(BoxPtr: Pointer; szPrefix: String): String;
Begin
  BoxHeader := BoxPtr;
  BoxEntry := BoxHeader^.BoxPos;
  While (BoxEntry<>NIL)AND(UpStr(szPrefix)<>UpStr(Tokenize(BoxEntry^.Content,1))) DO
    BoxEntry := BoxEntry^.Next;
  If (BoxEntry <> Nil) Then
  Begin
    BoxHeader^.BoxPos := BoxEntry^.Next;
    Result := Tokenize(BoxEntry^.Content,1);
  End
  else
  Begin
    BoxHeader^.BoxPos := BoxHeader^.First_Entry;
    Result := '';
  End;
End;

Function GetBoxString(BoxPtr: Pointer; KeyName: String; Default: String): String;
Begin
  BoxHeader := BoxPtr;
  BoxEntry := BoxHeader^.First_Entry;
  While (BoxEntry<>NIL)AND(UpStr(KeyName)<>UpStr(Tokenize(BoxEntry^.Content,1))) DO
    BoxEntry := BoxEntry^.Next;
  If (BoxEntry <> Nil) Then
    Result := Trim(Tokenize(BoxEntry^.Content,1))
  else
    Result := Default;
End;

Function GetBoxInt(BoxPtr: Pointer; KeyName: String; Default: Integer): Integer;
Var
  Str: String;

Begin
  Str := GetBoxString(BoxPtr, KeyName, 'NopE');
  If (Str <> 'NopE') Then
    Result := Str2Int(Str)
  else
    Result := Default;
End;

Function GetBoxBool(BoxPtr: Pointer; KeyName: String): Boolean;
Begin
  Result := GetBoxString(BoxPtr, KeyName, 'NopE')<> 'NopE';
End;

Function OpenBox(FileName: String;SubBoxStr: String): Pointer;

  Function SeekBox(SBoxStr: String):Integer;
  Var
    L: Integer;
    Str: String;
    Found: Boolean;
  Begin
    Found := false;L := 0;
    While NOT((EOF(BoxFile))Or Found)Do
    Begin
      Readln(BoxFile, Str); Inc(L);
      If ( Tokenize(Str,1)='#BEGIN' ) Then
        If ( Tokenize(Str,2)=SBoxStr ) Then
        Begin
          Found := True;
          SeekBox := L;
          Messageboxa(0,PChar(Str),'found',0);
        End;
    End;
    SeekBox := -1;
  End;

Var
  Line: Integer;
  Str: String;
  EOB: Boolean;

Begin

  EOB := false;
  Line := 0;
  AssignFile(BoxFile, FileName);
  {$I-}ReSet(BoxFile);{$I+}
  If (IOResult <> 0) then
    OpenBox := nil

  Else Begin

    If ( SubBoxStr<>'' ) Then
      Line := SeekBox(SubBoxStr);
    If ( Line=-1 ) Then
      Begin
        OpenBox := nil;
        CloseFile(BoxFile);
        Exit;
      End;

    New(BoxHeader);
    Boxheader^.FileName := FileName;
    Boxheader^.First_Entry := nil;
    New(BoxEntry);
    BoxEntry^.Next := Boxheader^.First_Entry;
    Boxheader^.First_Entry := BoxEntry;

    While NOT( (EOF(BoxFile))OR(EOB) )Do
    Begin
      Readln(BoxFile, Str);Str := Trim(Str);
      If ( (Str='')OR(Str[1]=';')OR(Str[1]='#') )Then
        Inc(Line)
      Else Begin
        Inc(Line);
        BoxEntry^.Content := Str;
        BoxEntry^.Line := Line;

        New(BoxEntry);
        BoxEntry^.Next := Boxheader^.First_Entry;
        Boxheader^.First_Entry := BoxEntry;
      End;
      If (Tokenize(Str,1)='#END') Then
        EOB := true;
    End;

    BoxHeader^.BoxPos := Boxheader^.First_Entry;
    OpenBox := BoxHeader;

  End;

  CloseFile(BoxFile);

End;


Procedure CloseBox(BoxPtr: pointer; Save: Boolean);
Begin
  BoxHeader := BoxPtr;


  If Save then
  Begin

  End
  Else Begin

    While Not(BoxEntry = nil)Do
    Begin
      BoxEntry := Boxheader^.First_Entry;
      Boxheader^.First_Entry := BoxEntry^.Next;
      Dispose(BoxEntry);
    End;

  End;

  Dispose(BoxHeader);
End;


end.
